/// smdi.cpp
///
/// (c) 2006-2008 Bastian Spiegel <bs@tkscript.de>
///     - Distributed under terms of the Lesser GNU General Public License (LGPL).
///       See COPYING and <http://www.gnu.org/licenses/licenses.html#LGPL> for further information.
///
///
/// changed: 09Sep2006, 04Nov2007, 02Jan2008, 13Jan2008
///


YG("smdi");


//#define DO_RESCAN_SCSI_BUS defined
//#define DO_SCSI_INQUIRY defined
#define DO_PRINT_SAMPLEHEADERS defined
//#define DO_SAMPLE_DOWNLOAD defined
//#define DO_SAMPLE_UPLOAD defined
//#define DO_MIDI_SYSEX defined // Note: SMDI MIDI is ignored by Yamaha A5000

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <windows.h>
#include <wnaspi32.h>


#define YAC_BIGSTRING defined
#include <yac.h>
#include <yac_host.cpp>
YAC_Host *yac_host;

#include "ying_smdi.h"
#include "ying_smdi.cpp"
#include "ying_smdi_SMDI.cpp"


// SCSI2 command IDs:
#define SCSI2_CMD_TESTUNITREADY 0x00
#define SCSI2_CMD_REQUESTSENSE  0x03
#define SCSI2_CMD_RECEIVE       0x08
#define SCSI2_CMD_SEND          0x0A
#define SCSI2_CMD_INQUIRY       0x12

// SCSI Architecture Model (SAM) status codes (taken from linux scsi.h):
#define SAM_STAT_GOOD            0x00
#define SAM_STAT_CHECK_CONDITION 0x02
#define SAM_STAT_CONDITION_MET   0x04
#define SAM_STAT_BUSY            0x08
#define SAM_STAT_INTERMEDIATE    0x10
#define SAM_STAT_INTERMEDIATE_CONDITION_MET 0x14
#define SAM_STAT_RESERVATION_CONFLICT 0x18
#define SAM_STAT_COMMAND_TERMINATED 0x22        /* obsolete in SAM-3 */
#define SAM_STAT_TASK_SET_FULL   0x28
#define SAM_STAT_ACA_ACTIVE      0x30
#define SAM_STAT_TASK_ABORTED    0x40

// ASPI function types:
typedef DWORD (*proc_t_GetASPI32SupportInfo)  ( void ) ;
typedef DWORD (*proc_t_SendASPI32Command)      ( LPSRB );
typedef BOOL  (*proc_t_GetASPI32Buffer)       ( PASPI32BUFF );
typedef BOOL  (*proc_t_FreeASPI32Buffer)      ( PASPI32BUFF );
typedef BOOL  (*proc_t_TranslateASPI32Address)( PDWORD, PDWORD );

// Resove ASPI function pointers:
static proc_t_GetASPI32SupportInfo   proc_GetASPI32SupportInfo;
static proc_t_SendASPI32Command      proc_SendASPI32Command;
static proc_t_GetASPI32Buffer        proc_GetASPI32Buffer;
static proc_t_FreeASPI32Buffer       proc_FreeASPI32Buffer;
static proc_t_TranslateASPI32Address proc_TranslateASPI32Address;

// SMDI reject (sub-) codes:
#define SMDI_REJECTCODE_REJECTED 0x0002
#define SMDI_REJECTSUBCODE_REJECTED_PROCEDURENOTSUPPORTED 0x0000
#define SMDI_REJECTSUBCODE_REJECTED_INAPPROPRIATE         0x0002

#define SMDI_REJECTCODE_BUSY 0x0005

#define SMDI_REJECTCODE_SENDNEXTPACKET 0x0011
#define SMDI_REJECTSUBCODE_SENDNEXTPACKET_PACKETNUMBERMISMATCH 0x0000

#define SMDI_REJECTCODE_SAMPLE 0x0020
#define SMDI_REJECTSUBCODE_SAMPLE_SAMPLENUMBEROUTOFRANGE   0x0000
#define SMDI_REJECTSUBCODE_SAMPLE_NOSAMPLEATSAMPLENUMBER   0x0002
#define SMDI_REJECTSUBCODE_SAMPLE_INSUFFICIENTSAMPLEMEMORY 0x0004
#define SMDI_REJECTSUBCODE_SAMPLE_INSUFFICIENTPARAMMEMORY  0x0005
#define SMDI_REJECTSUBCODE_SAMPLE_UNKNOWNBITSPERWORD       0x0006
#define SMDI_REJECTSUBCODE_SAMPLE_UNKNOWNNUMBEROFCHANNELS  0x0007

#define SMDI_REJECTCODE_BEGINSAMPLETRANSFER 0x0022
#define SMDI_REJECTSUBCODE_BEGINSAMPLETRANSFER_SAMPLENUMBERMISMATCH 0x0001
#define SMDI_REJECTSUBCODE_BEGINSAMPLETRANSFER_DATAPACKETLENGTH     0x0002

// SMDI sense codes: (CHECK_CONDITION)
#define SMDI_SENSECODE_SENDREJECTED_SLAVEREPLYMESSAGEPENDING 0x80
#define SMDI_SENSECODE_RECEIVEREJECTED_NOSLAVEREPLYMESSAGEPENDING 0x81
#define SMDI_SENSECODE_SENDREJECTED_WRONGINITIATORTRANSFERLENGTH 0x82 // Smaller than 11
#define SMDI_SENSECODE_RECEIVEREJECTED_WRONGINITIATORALLOCLENGTH 0x83
#define SMDI_SENSECODE_SENDREJECTED_NONSMDIMESSAGE 0x84 // Initiator is sending a non-SMDI message
#define SMDI_SENSECODE_SENDREJECTED_TRANSFERLENGTHANDADDITIONALMESSAGELENGTHCONFLICT 0x85
#define SMDI_SENSECODE_SENDREJECTED_INCORRECTADDITIONALMESSAGELENGTH 0x86

// SMDI message IDs:
#define SMDI_MSGID_IDENTIFY               0x0001
#define SMDI_MSGID_REJECT                 0x0002
#define SMDI_MSGID_ACK                    0x0100
#define SMDI_MSGID_NAK                    0x0101
#define SMDI_MSGID_WAIT                   0x0102
#define SMDI_MSGID_SENDNEXTPACKET         0x0103
#define SMDI_MSGID_ENDOFPROCEDURE         0x0104
#define SMDI_MSGID_ABORTPROCEDURE         0x0105
#define SMDI_MSGID_DATAPACKET             0x0110
#define SMDI_MSGID_SAMPLEHEADERREQUEST    0x0120
#define SMDI_MSGID_SAMPLEHEADER           0x0121
#define SMDI_MSGID_BEGINSAMPLETRANSFER    0x0122
#define SMDI_MSGID_SAMPLENAME             0x0123
#define SMDI_MSGID_DELETESAMPLEFROMMEMORY 0x0124
#define SMDI_MSGID_TRANSMITMIDIMESSAGE    0x0200

// SMDI message structures:
typedef struct __SMDI_Identify { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x00 0x01
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_Identify; // 11 bytes, used for Master + Slave Identify messages

typedef struct __SMDI_Reject {
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x00 0x02
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x04
   BYTE rejection_code[2]; // 
   BYTE rejection_sub_code[2]; // 
} SMDI_Reject; // 15 bytes

typedef struct __SMDI_Ack { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x00
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_Ack; // 11 bytes

typedef struct __SMDI_Nak { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x01
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_Nak; // 11 bytes

typedef struct __SMDI_Wait { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x02
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_Wait; // 11 bytes, ~250ms

typedef struct __SMDI_SendNextPacket { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x03
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x03
   BYTE packet_nr[3];
} SMDI_SendNextPacket; // 14 bytes

typedef struct __SMDI_EndOfProcedure { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x04
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_EndOfProcedure; // 11 bytes, Signalize normal completition of a procedure

typedef struct __SMDI_AbortProcedure { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x05
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00
} SMDI_AbortProcedure; // 11 bytes, Halt procedure immediately

typedef struct __SMDI_DataPacket { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x10
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x03+n
   BYTE packet_nr[3];
   // Data bytes follow
} SMDI_DataPacket; // 14+n bytes, Sample are left-justified, MSB first, multi-channel interleaved

typedef struct __SMDI_SampleHeaderRequest { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x20
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x03
   BYTE sample_nr[3];
} SMDI_SampleHeaderRequest; // 14 bytes

typedef struct __SMDI_SampleHeader { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x21
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x1A+n
   BYTE sample_nr[3];
   BYTE sample_bits_per_word;
   BYTE sample_num_channels;
   BYTE sample_period[3]; // nanoseconds
   BYTE sample_length[4]; // #words
   BYTE sample_loop_start[4]; // word offset
   BYTE sample_loop_end[4];   // word offset
   BYTE sample_loop_ctrl;  // 0x00=Forward only, 0x01=Backward/Forward, 0x7F=Loop off
   BYTE sample_pitch_integer[2]; // orig key, default=0x003C (middle C)
   BYTE sample_pitch_fraction[2]; // semi-tone up, 0x8000 = 50% up
   BYTE sample_name_length; // (n)
   // Sample name chars follow
} SMDI_SampleHeader; // 37+n bytes, 

typedef struct __SMDI_SampleName { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x23
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x04+n
   BYTE sample_nr[3];
   BYTE sample_name_length;
} SMDI_SampleName; // 15+n bytes, Change sample name, Expect EndOfProcedure reply

typedef struct __SMDI_DeleteSampleFromMemory { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x24
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x03
   BYTE sample_nr[3];
} SMDI_DeleteSampleFromMemory; // 14 bytes, Delete sample, Expect EndOfProcedure reply (prepare for Wait!)

typedef struct __SMDI_BeginSampleTransfer { // MSB first!, Also used for BeginSampleTransferAcknowledge
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x22
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x06
   BYTE sample_nr[3];
   BYTE data_packet_len[3]; // Slave->Master: BeginSampleTransfer: Req:Max. length Ack:Bytes per packet
                            // Slave->Master: SampleHeader: Ack: Max. number of bytes per packet that slave can handle
                            // Master->Slave: Data packet length actually used (bytes)
} SMDI_BeginSampleTransfer; // 17 bytes, Must be preceded by SampleHeader message. Expected Slave Reply: SendNextPacket

typedef struct __SMDI_TransmitMIDIMessage { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x02 0x00
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x00+n
} SMDI_TransmitMIDIMessage; // 11+n bytes

// Time to wait (millisec) until sampler becomes ready for data transfer
#define SMDI_TESTUNITREADY_TIMEOUT 25000

static HANDLE event_handle;
static HINSTANCE dll_handle;
static BYTE adapter_nr = 0; // default adapter nr
static BYTE target_nr = 6;  // default target (SCSI ID)

static unsigned char smdi_transfer_buffer[64*1024 + sizeof(SMDI_DataPacket)];


// ---------------------------------------------------------------------------- aspi_status_to_string
static const char *aspi_status_to_string(BYTE _status) {
   // Convert SRB_Status byte to string
   switch(_status)
   {
   default:                        return "<unknown ASPI status>";
   case SS_PENDING:                return "SS_PENDING";
   case SS_COMP:                   return "SS_COMP";
   case SS_ABORTED:                return "SS_ABORTED";
   case SS_ERR:                    return "SS_ERR";
   case SS_INVALID_CMD:            return "SS_INVALID_CMD";
   case SS_INVALID_HA:             return "SS_INVALID_HA";
   case SS_NO_DEVICE:              return "SS_NO_DEVICE";
   case SS_INVALID_SRB:            return "SS_INVALID_SRB";
   case SS_BUFFER_ALIGN:           return "SS_BUFFER_ALIGN";
   case SS_ILLEGAL_MODE:           return "SS_ILLEGAL_MODE";
   case SS_NO_ASPI:                return "SS_NO_ASPI";
   case SS_FAILED_INIT:            return "SS_FAILED_INIT";
   case SS_ASPI_IS_BUSY:           return "SS_ASPI_IS_BUSY";
   case 0xE6:/*SS_BUFFER_TOO_BIG:*/return "SS_BUFFER_TOO_BIG";
   case SS_MISMATCHED_COMPONENTS:  return "SS_MISMATCHED_COMPONENTS";
   case SS_NO_ADAPTERS :           return "SS_NO_ADAPTERS ";
   case SS_INSUFFICIENT_RESOURCES: return "SS_INSUFFICIENT_RESOURCES";
   }
}


// ---------------------------------------------------------------------------- get_hostadapter_info2
static int get_hostadapter_info2(SRB_HAInquiry *inq, unsigned char _haId) {
   memset(inq, 0, sizeof(SRB_HAInquiry));
   inq->SRB_Cmd = SC_HA_INQUIRY;
   inq->SRB_HaId = _haId;
   //ResetEvent(event_handle);
   proc_SendASPI32Command(inq);
   /*if(inq->SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, INFINITE);
   }*/
   if(inq->SRB_Status == SS_COMP)
   {
      // Get Info completed
      return 1;
   }
   else
   {
      printf("[---] tksmdi: SRB_HAInquiry(haId=%i) failed (%s).\n", _haId, aspi_status_to_string(inq->SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- get_hostadapter_info
static int get_hostadapter_info(void) {
   SRB_HAInquiry inq;
   BYTE numHostAdapters;
   BYTE i;

   if(get_hostadapter_info2(&inq, 0))
   {
      numHostAdapters = inq.HA_Count;
      printf("[...] tksmdi: #hostadapters=%i.\n", numHostAdapters);
      for(i=0; i<numHostAdapters; i++)
      {
         get_hostadapter_info2(&inq, i);
         printf("[...] tksmdi: hostadapter[%i].HA_ManagerID = \"%.16s\"\n", i, inq.HA_ManagerId);
         printf("[...] tksmdi: hostadapter[%i].HA_Identifier = \"%.16s\"\n", i, inq.HA_Identifier);
         printf("[...] tksmdi: hostadapter[%i].HA_SCSI_ID = %i\n", i, inq.HA_SCSI_ID);
      }
      return 1;
   }
   // Failed
   return 0;
}


// ---------------------------------------------------------------------------- rescan_scsi_bus
static int rescan_scsi_bus(BYTE _adapterNr) {
   SRB_RescanPort srb;

   memset(&srb, 0, sizeof(SRB_RescanPort));
   srb.SRB_Cmd = SC_RESCAN_SCSI_BUS;
   srb.SRB_HaId = _adapterNr;
   //ResetEvent(event_handle);
   //srb.SRB_PostProc = event_handle;
   proc_SendASPI32Command(&srb);
   /*if(srb.SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, 15000); // Wait max. 15s
   }*/
   if(srb.SRB_Status == SS_COMP)
   {
      // Get Info completed
      return 1;
   }
   else
   {
      printf("[---] tksmdi: SC_RESCAN_SCSI_BUS(haId=%i) failed (%s).\n", _adapterNr, aspi_status_to_string(srb.SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- scsi2_inquiry
int scsi2_inquiry(BYTE _adapterNr, BYTE _target) {
   SRB_ExecSCSICmd exec;
   BYTE result[96];

   memset(&exec, 0, sizeof(SRB_ExecSCSICmd));
   exec.SRB_Cmd        = SC_EXEC_SCSI_CMD;
   exec.SRB_HaId       = _adapterNr;
   exec.SRB_Target     = _target; // SCSI ID
   exec.SRB_Flags      = SRB_DIR_IN | SRB_EVENT_NOTIFY; // target->host data transfer follows
   exec.SRB_BufLen     = 96; // Receive buffer len
   exec.SRB_BufPointer = result;
   exec.SRB_SenseLen   = SENSE_LEN;
   exec.SRB_CDBLen     = 6; // SCSI Command Descriptor Block len (6,10,12 bytes)
   exec.CDBByte[0]     = SCSI2_CMD_INQUIRY; // Operation code (INQUIRY)
   exec.CDBByte[4]     = 96;   // Allocation length
   ResetEvent(event_handle);
   exec.SRB_PostProc   = event_handle;
   proc_SendASPI32Command(&exec);
   if(exec.SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, 15000); // Wait max. 15s
   }
   if(exec.SRB_Status == SS_COMP)
   {
      // Get Info completed
      printf("[...] tksmdi: INQUIRY(haId=%i Target=%i) OK:\n"
         "\t\t      Vendor = \"%.8s\"\n"
         "\t\t     Product = \"%.16s\"\n"
         "\t\t Device type = %08x\n", // Type should be 0x03 for SMDI devices
         _adapterNr, _target, result+8, result+16, result[0]&31
         );
      return 1;
   }
   else
   {
      printf("[---] tksmdi: INQUIRY(haId=%i Target=%i) failed (%s).\n", _adapterNr, _target, aspi_status_to_string(exec.SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_get_uint16
static unsigned short smdi_get_uint16(BYTE *_buf) {
   return 
      (((unsigned short)_buf[0])<<8) | 
      ((unsigned short)_buf[1]) ;
}

// ---------------------------------------------------------------------------- smdi_get_uint24
static unsigned short smdi_get_uint24(BYTE *_buf) {
   return 
      (((unsigned short)_buf[0])<<16) | 
      (((unsigned short)_buf[1])<< 8) | 
      ((unsigned short)_buf[2]) ;
}

// ---------------------------------------------------------------------------- smdi_get_uint32
static unsigned short smdi_get_uint32(BYTE *_buf) {
   return 
      (((unsigned short)_buf[0])<<24) | 
      (((unsigned short)_buf[1])<<16) | 
      (((unsigned short)_buf[2])<< 8) | 
      ((unsigned short)_buf[3]) ;
}

// ---------------------------------------------------------------------------- smdi_set_uint16
static void smdi_set_uint16(BYTE *_buf, unsigned short _v) {
   _buf[0] = (_v>> 8)&0xFF;
   _buf[1] = (_v    )&0xFF;
}

// ---------------------------------------------------------------------------- smdi_set_uint24
static void smdi_set_uint24(BYTE *_buf, unsigned int _v) {
   _buf[0] = (_v>>16)&0xFF;
   _buf[1] = (_v>> 8)&0xFF;
   _buf[2] = (_v    )&0xFF;
}

// ---------------------------------------------------------------------------- smdi_set_uint32
static void smdi_set_uint32(BYTE *_buf, unsigned int _v) {
   _buf[0] = (_v>>24)&0xFF;
   _buf[1] = (_v>>16)&0xFF;
   _buf[2] = (_v>> 8)&0xFF;
   _buf[3] = (_v    )&0xFF;
}

// ---------------------------------------------------------------------------- smdi_send_message
int smdi_send_message(BYTE _adapterNr, BYTE _target, BYTE *_msg, unsigned int _msgLen, 
                      unsigned short _msgId, unsigned int _addMsgLen) 
{
   SRB_ExecSCSICmd exec;

   _msg[0] = 'S';
   _msg[1] = 'M';
   _msg[2] = 'D';
   _msg[3] = 'I';
   smdi_set_uint16(_msg+4, _msgId);
   smdi_set_uint24(_msg+8, _addMsgLen);

   memset(&exec, 0, sizeof(SRB_ExecSCSICmd));
   exec.SRB_Cmd        = SC_EXEC_SCSI_CMD;
   exec.SRB_HaId       = _adapterNr;
   exec.SRB_Target     = _target; // SCSI ID
   exec.SRB_Flags      = SRB_DIR_OUT | SRB_EVENT_NOTIFY; // host->target data transfer follows
   exec.SRB_BufLen     = _msgLen; // Send buffer len
   exec.SRB_BufPointer = _msg;
   exec.SRB_SenseLen   = SENSE_LEN;
   exec.SRB_CDBLen     = 6; // SCSI Command Descriptor Block len (6,10,12 bytes)
   exec.CDBByte[0]     = SCSI2_CMD_SEND; // Operation code (SEND)
   exec.CDBByte[2]     = (_msgLen>>16)&0xFF; // Transfer length 24..16
   exec.CDBByte[3]     = (_msgLen>> 8)&0xFF; // Transfer length 15..8
   exec.CDBByte[4]     = (_msgLen    )&0xFF; // Transfer length  7..0
   ResetEvent(event_handle);
   exec.SRB_PostProc   = event_handle;
   proc_SendASPI32Command(&exec);
   if(exec.SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, 15000); // Wait max. 15s
   }
   if(exec.SRB_Status == SS_COMP)
   {
      // Send request completed
      printf("[...] tksmdi: smdi_send_message (haId=%i Target=%i) sent OK.\n",
         _adapterNr, _target
         );
      return 1;
   }
   else
   {
      printf("[---] tksmdi: smdi_send_message (haId=%i Target=%i) FAILED (%s).\n", 
         _adapterNr, _target, aspi_status_to_string(exec.SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_receive_message
int smdi_receive_message(BYTE _adapterNr, BYTE _target, BYTE *_msg, unsigned int _msgLen) {
   SRB_ExecSCSICmd exec;
   
   memset(&exec, 0, sizeof(SRB_ExecSCSICmd));
   exec.SRB_Cmd        = SC_EXEC_SCSI_CMD;
   exec.SRB_HaId       = _adapterNr;
   exec.SRB_Target     = _target; // SCSI ID
   exec.SRB_Flags      = SRB_DIR_IN | SRB_EVENT_NOTIFY; // host->target data transfer follows
   exec.SRB_BufLen     = _msgLen; // Receive buffer len
   exec.SRB_BufPointer = _msg;
   exec.SRB_SenseLen   = SENSE_LEN;
   exec.SRB_CDBLen     = 6; // SCSI Command Descriptor Block len (6,10,12 bytes)
   exec.CDBByte[0]     = SCSI2_CMD_RECEIVE; // Operation code (RECEIVE)
   exec.CDBByte[2]     = (_msgLen>>16)&0xFF; // Receive length 24..16
   exec.CDBByte[3]     = (_msgLen>> 8)&0xFF; // Receive length 15..8
   exec.CDBByte[4]     = (_msgLen    )&0xFF; // Receive length  7..0
   ResetEvent(event_handle);
   exec.SRB_PostProc   = event_handle;
   proc_SendASPI32Command(&exec);
   if(exec.SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, 15000); // Wait max. 15s
   }
   if(exec.SRB_Status == SS_COMP)
   {
      // Receive message completed
      printf("[...] tksmdi: smdi_receive_message (haId=%i Target=%i len=%i) received OK.\n",
         _adapterNr, _target, _msgLen
         );
      
      return 1;
   }
   else
   {
      printf("[---] tksmdi: smdi_receive_message (haId=%i Target=%i len=%i) FAILED (%s).\n", 
         _adapterNr, _target, _msgLen, aspi_status_to_string(exec.SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_test_unit_ready
int smdi_test_unit_ready(BYTE _adapterNr, BYTE _target) {
   SRB_ExecSCSICmd exec;
   unsigned char buf;

   memset(&exec, 0, sizeof(SRB_ExecSCSICmd));
   exec.SRB_Cmd        = SC_EXEC_SCSI_CMD;
   exec.SRB_HaId       = _adapterNr;
   exec.SRB_Target     = _target; // SCSI ID
   exec.SRB_Flags      = SRB_EVENT_NOTIFY; // host->target data transfer follows
   exec.SRB_BufLen     = 1; // Receive buffer len
   exec.SRB_BufPointer = &buf;
   exec.SRB_SenseLen   = SENSE_LEN;
   exec.SRB_CDBLen     = 6; // SCSI Command Descriptor Block len (6,10,12 bytes)
   exec.CDBByte[0]     = SCSI2_CMD_TESTUNITREADY; // Operation code (TEST_UNIT_READY)
   ResetEvent(event_handle);
   exec.SRB_PostProc   = event_handle;
   proc_SendASPI32Command(&exec);
   if(exec.SRB_Status == SS_PENDING)
   {
      WaitForSingleObject(event_handle, 15000); // Wait max. 15s
   }
   if(exec.SRB_Status == SS_COMP)
   {
      // Bit0 is reserved in SCSI-2
      return ((buf&0xFE) == SAM_STAT_GOOD);
   }
   else
   {
      printf("[---] tksmdi: smdi_test_unit_ready (haId=%i Target=%i) FAILED (%s).\n", 
         _adapterNr, _target, aspi_status_to_string(exec.SRB_Status));
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_wait_unit_ready
int smdi_wait_unit_ready(BYTE _adapterNr, BYTE _target, unsigned int _timeoutMS) {
   int waitCount = ((int)_timeoutMS+249) / 250;

   while(!smdi_test_unit_ready(_adapterNr, _target))
   {
      if(waitCount--<0)
         return 0;
      Sleep(250);
   }
   return 1;
}


// ---------------------------------------------------------------------------- smdi_get_sampleheader
int smdi_get_sampleheader(BYTE _adapterNr, BYTE _target, unsigned int _sampleNr, BYTE *_sampleHeader, unsigned int _sampleHeaderSize) {
/*
 Sample receive (slave->host): 
    out: Sample Header Request Message
     in: OK: Sample Header Reply
     in: ERR: SMDI Message Reject Reply
    out: Begin Sample Transfer Message
     in: OK: Begin Sample Transfer Acknowledge
         ERR: SMDI Message Reject
    out: Send Next Packet
     in: OK: Data Packet Message
         ERR: SMDI Message Reject (packet# mismatch)
         ERR: Abort Procedure Message (internal error, manual override)
     ... Final Data Packet 
   */

   SMDI_SampleHeaderRequest shq;

/*
typedef struct __SMDI_SampleHeaderRequest { // MSB first!
   BYTE tag[4];         // 'S' 'M' 'D' 'I'
   BYTE id[2];          // 0x01 0x20
   BYTE sub_id[2];      // 0x00 x00
   BYTE add_msg_len[3]; // 0x00 0x00 0x03
   BYTE sample_nr[3];
} SMDI_SampleHeaderRequest; // 14 bytes
*/
   memset(&shq, 0, sizeof(SMDI_SampleHeaderRequest));
   smdi_set_uint24(shq.sample_nr, _sampleNr);

   if(smdi_send_message(_adapterNr, _target, (BYTE*)&shq, sizeof(SMDI_SampleHeaderRequest), SMDI_MSGID_SAMPLEHEADERREQUEST, 0x03))
   {
      // Send request completed
      printf("[...] tksmdi: SampleHeaderRequest (haId=%i Target=%i) sent OK.\n",
         _adapterNr, _target
         );

      if(smdi_receive_message(_adapterNr, _target, (BYTE*)_sampleHeader, _sampleHeaderSize))
      {
         if( smdi_get_uint16(_sampleHeader+4) == SMDI_MSGID_SAMPLEHEADER)
         {
            printf("[...] tksmdi: SampleHeader (haId=%i Target=%i) received OK.\n",
               _adapterNr, _target
               );
            return 1;
         }
         else
         {
            printf("[---] tksmdi: smdi_get_sampleheader(sampleNr=%i) failed, reply message id is 0x%04x\n",
               _sampleNr, smdi_get_uint16(_sampleHeader+4));
            return 0;
         }
      }
      else
      {
         printf("[---] tksmdi: Receive SampleHeader (haId=%i Target=%i sampleNr=%i) FAILED.\n", 
            _adapterNr, _target, _sampleNr);
         return 0;
      }
      return 1;
   }
   else
   {
      printf("[---] tksmdi: Send SampleHeaderRequest (haId=%i Target=%i sampleNr=%i) FAILED.\n", 
         _adapterNr, _target, _sampleNr);
      return 0;
   }

}


// ---------------------------------------------------------------------------- smdi_print_sampleheader
int smdi_print_sampleheader(BYTE _adapterNr, BYTE _target, unsigned int _sampleNr) {
   BYTE sampleHeaderData[256+sizeof(SMDI_SampleHeader)];
   SMDI_SampleHeader *sampleHeader;

   sampleHeader = (SMDI_SampleHeader*) sampleHeaderData;
   memset(sampleHeaderData, 0xCD, 256+sizeof(SMDI_SampleHeader));
   if(smdi_get_sampleheader(_adapterNr, _target, _sampleNr, sampleHeaderData, 256+sizeof(SMDI_SampleHeader)))
   {
      static char nameFmt[32];

      printf("[...] tksmdi: Sample header:\n"
         "\t\tSample number = %i\n"
         "\t\tBits per word = %i\n"
         "\t\t    #channels = %i\n"
         "\t\tSample period = %i\n"
         "\t\tSample length = %i\n"
         "\t\t   Loop start = %i\n"
         "\t\t     Loop end = %i\n"
         "\t\t    Loop ctrl = %i\n"
         "\t\t  Pitch (int) = %i\n"
         "\t\t Pitch (frac) = %i\n"
         "\t\t  Name length = %i\n"
         , smdi_get_uint24(sampleHeader->sample_nr)
         , sampleHeader->sample_bits_per_word
         , sampleHeader->sample_num_channels
         , smdi_get_uint24(sampleHeader->sample_period)
         , smdi_get_uint32(sampleHeader->sample_length)
         , smdi_get_uint32(sampleHeader->sample_loop_start)
         , smdi_get_uint32(sampleHeader->sample_loop_end)
         , sampleHeader->sample_loop_ctrl
         , smdi_get_uint16(sampleHeader->sample_pitch_integer)
         , smdi_get_uint16(sampleHeader->sample_pitch_fraction)
         , sampleHeader->sample_name_length
         );
      if(sampleHeader->sample_name_length)
      {
         sprintf(nameFmt, "\t\t         Name = \"%%.%is\"\n", sampleHeader->sample_name_length);
         printf(nameFmt, (const char*) (sampleHeader+1));
      }

      return 1;

   }
   return 0;
}

// ---------------------------------------------------------------------------- smdi_download_sample
int smdi_download_sample(BYTE _adapterNr, BYTE _target, unsigned int _sampleNr) {
   BYTE sampleHeaderData[256+sizeof(SMDI_SampleHeader)];
   SMDI_SampleHeader *sampleHeader;
   SMDI_BeginSampleTransfer bst;
   DWORD tickCountStart, tickCountEnd;

   sampleHeader = (SMDI_SampleHeader*) sampleHeaderData;
   memset(sampleHeaderData, 0xCD, 256+sizeof(SMDI_SampleHeader));
   if(smdi_get_sampleheader(_adapterNr, _target, _sampleNr, sampleHeaderData, 256+sizeof(SMDI_SampleHeader)))
   {
      static char nameFmt[32];
      unsigned int sampleByteSize;

      sampleByteSize = ((sampleHeader->sample_bits_per_word+7)/8); // Bytes per sample
      sampleByteSize *= sampleHeader->sample_num_channels * smdi_get_uint32(sampleHeader->sample_length);

      printf("[...] tksmdi: Sample header:\n"
         "\t\tSample number = %i\n"
         "\t\t Size (bytes) = %i\n"
         "\t\tBits per word = %i\n"
         "\t\t    #channels = %i\n"
         "\t\tSample period = %i\n"
         "\t\tSample length = %i\n"
         "\t\t   Loop start = %i\n"
         "\t\t     Loop end = %i\n"
         "\t\t    Loop ctrl = %i\n"
         "\t\t  Pitch (int) = %i\n"
         "\t\t Pitch (frac) = %i\n"
         "\t\t  Name length = %i\n"
         , smdi_get_uint24(sampleHeader->sample_nr)
         , sampleByteSize
         , sampleHeader->sample_bits_per_word
         , sampleHeader->sample_num_channels
         , smdi_get_uint24(sampleHeader->sample_period)
         , smdi_get_uint32(sampleHeader->sample_length)
         , smdi_get_uint32(sampleHeader->sample_loop_start)
         , smdi_get_uint32(sampleHeader->sample_loop_end)
         , sampleHeader->sample_loop_ctrl
         , smdi_get_uint16(sampleHeader->sample_pitch_integer)
         , smdi_get_uint16(sampleHeader->sample_pitch_fraction)
         , sampleHeader->sample_name_length
         );
      if(sampleHeader->sample_name_length)
      {
         sprintf(nameFmt, "\t\t         Name = \"%%.%is\"\n", sampleHeader->sample_name_length);
         printf(nameFmt, (const char*) (sampleHeader+1));
      }

      // Profile download speed
      tickCountStart = GetTickCount();

      // Send BeginSampleTransfer message
      memset(&bst, 0, sizeof(SMDI_BeginSampleTransfer));
      smdi_set_uint24(bst.sample_nr, _sampleNr);
      smdi_set_uint24(bst.data_packet_len, 64*1024); // Host can handle up to 64kb/packet
      if(smdi_send_message(_adapterNr, _target, (BYTE*)&bst, sizeof(SMDI_BeginSampleTransfer), SMDI_MSGID_BEGINSAMPLETRANSFER, 0x06))
      {
         memset(&bst, 0, sizeof(SMDI_BeginSampleTransfer));
         // Expect BeginSampleTransferAcknowledge or MessageReject
         if(smdi_receive_message(_adapterNr, _target, (BYTE*)&bst, sizeof(SMDI_BeginSampleTransfer)))
         {
            if(smdi_get_uint16(bst.id) == SMDI_MSGID_BEGINSAMPLETRANSFER)
            {
               SMDI_SendNextPacket snp;
               unsigned int dataPacketLen;
               unsigned int dataPacketNr;
               unsigned int numBytesTransferred;

               dataPacketLen = smdi_get_uint24(bst.data_packet_len);
               printf("[...] tksmdi: download data packet len = %i\n", dataPacketLen);
               if( (dataPacketLen > 65536) || (dataPacketLen==0) )
               {
                  printf("[---] tksmdi: smdi_download_sample: dataPacketLen(%i) is not supported by host.\n",
                     dataPacketLen);
                  return 0;
               }

               // Transfer sample packets
               dataPacketNr = 0;
               numBytesTransferred = 0;
               while(numBytesTransferred < sampleByteSize)
               {
                  // Send SendNextPacket message to sampler
                  memset(&snp, 0, sizeof(SMDI_SendNextPacket));
                  smdi_set_uint24(snp.packet_nr, dataPacketNr);
                  if(smdi_send_message(_adapterNr, _target, (BYTE*)&snp, sizeof(SMDI_SendNextPacket), SMDI_MSGID_SENDNEXTPACKET, 0x03))
                  {
                     if(smdi_receive_message(_adapterNr, _target, smdi_transfer_buffer, sizeof(SMDI_DataPacket)+dataPacketLen))
                     {
                        numBytesTransferred += dataPacketLen;
                        dataPacketNr++;
                        // xxx copy sample data to destination buffer..
                     }
                     else
                     {
                        printf("[---] tksmdi: smdi_download_sample: failed to receive DataPacket (dataPacketNr=%i).\n",
                           dataPacketNr);
                        return 0;
                     }
                  }
                  else
                  {
                     printf("[---] tksmdi: smdi_download_sample: failed to send SendNextPacket message (dataPacketNr=%i).\n",
                        dataPacketNr);
                     return 0;
                  }

               }
               // OK
               {
                  double bps;
                  tickCountEnd = GetTickCount();
                  bps = ((double)sampleByteSize) / (tickCountEnd-tickCountStart);
                  printf("[...] tksmdi: sample download complete (%i bytes transferred; %f kb/s).\n", 
                     sampleByteSize, bps*1000.0/1024.0);
               }
               return 1;
            }
            else
            {
               printf("[---] tksmdi: BeginSampleTransfer not acknowledged.\n");
            }
         }
         else
         {
            printf("[---] tksmdi: failed to receive BeginSampleTransferAcknowledge (adapterNr=%i target=%i).\n",
               _adapterNr, _target);
         }
      }
      else
      {
         printf("[---] tksmdi: failed to send BeginSampleTransfer message (adapterNr=%i target=%i).\n",
            _adapterNr, _target);
      }
   }
   return 0;
}


// ---------------------------------------------------------------------------- smdi_upload_sample
int smdi_upload_sample(BYTE _adapterNr, BYTE _target, unsigned int _sampleNr,
                       BYTE _bitsPerWord, BYTE _numChannels, 
                       unsigned int _sampleRateNanoSec,
                       unsigned int _sampleLength,
                       unsigned int _loopStart,
                       unsigned int _loopEnd,
                       BYTE _loopCtrl,
                       unsigned short _origKeyInt,
                       unsigned short _origKeyFrac,
                       const char *_sampleName,
                       unsigned char *_sampleData, // Left justified, MSB first, channel interleaved
                       int _bDataIsLittleEndian    // Pass 1 to swap byte order during upload
                       )
{
   BYTE sampleHeaderData[256+sizeof(SMDI_SampleHeader)];
   SMDI_SampleHeader *sampleHeader;
   SMDI_BeginSampleTransfer bst;
   DWORD tickCountStart, tickCountEnd;
   int sampleNameLen;
   unsigned int sampleByteSize;
   unsigned int bytesPerWord;
   unsigned int dataPacketLen;

   // Profile upload speed
   tickCountStart = GetTickCount();


   sampleHeader = (SMDI_SampleHeader*) sampleHeaderData;
   memset(sampleHeaderData, 0x00, 256+sizeof(SMDI_SampleHeader));
   smdi_set_uint24(sampleHeader->sample_nr, _sampleNr);
   sampleHeader->sample_bits_per_word = _bitsPerWord;
   sampleHeader->sample_num_channels = _numChannels;
   smdi_set_uint24(sampleHeader->sample_period, _sampleRateNanoSec);
   smdi_set_uint32(sampleHeader->sample_length, _sampleLength);
   smdi_set_uint32(sampleHeader->sample_loop_start, _loopStart);
   smdi_set_uint32(sampleHeader->sample_loop_end, _loopEnd);
   sampleHeader->sample_loop_ctrl = _loopCtrl;
   smdi_set_uint16(sampleHeader->sample_pitch_integer, _origKeyInt);
   smdi_set_uint16(sampleHeader->sample_pitch_fraction, _origKeyFrac);
   sampleNameLen = strlen(_sampleName);
   if(sampleNameLen>255)
      sampleNameLen=255;
   sampleHeader->sample_name_length = (BYTE) sampleNameLen;
   strcpy((char*) (sampleHeader+1), _sampleName);
   bytesPerWord = (_bitsPerWord+7)/8;
   sampleByteSize = bytesPerWord * _sampleLength * _numChannels;

   if(smdi_send_message(_adapterNr, _target, sampleHeaderData, sizeof(SMDI_SampleHeader)+sampleNameLen, SMDI_MSGID_SAMPLEHEADER, 26+sampleNameLen))
   {
      printf("[...] tksmdi: smdi_upload_sample: SampleHeader sent OK.\n");
      // Expect BeginSampleTransferAcknowledge
      if(smdi_receive_message(_adapterNr, _target, (BYTE*)&bst, sizeof(SMDI_BeginSampleTransfer)))
      {
         if(smdi_get_uint16(bst.id) == SMDI_MSGID_BEGINSAMPLETRANSFER)
         {
            dataPacketLen = smdi_get_uint24(bst.data_packet_len);
            printf("[...] tksmdi: smdi_upload_sample: BeginSampleTransferAcknowledge received. DataPacketLen=%i\n",
               dataPacketLen);
            if( (dataPacketLen>65536) || (dataPacketLen==0) )
            {
               printf("[---] tksmdi: smdi_upload_sample: dataPacketLen(%i) is not supported by host.\n", dataPacketLen);
            }
            else
            {
               memset(&bst, 0, sizeof(SMDI_BeginSampleTransfer));
               smdi_set_uint24(bst.sample_nr, _sampleNr); 
               smdi_set_uint24(bst.data_packet_len, dataPacketLen); 
               if(smdi_send_message(_adapterNr, _target, (BYTE*)&bst, sizeof(SMDI_BeginSampleTransfer), SMDI_MSGID_BEGINSAMPLETRANSFER, 0x06))
               {
                  SMDI_SendNextPacket snp;
                  unsigned int dataPacketNr;
                  unsigned int currentPacketLen;
                  unsigned int numBytesTransferred;
                  unsigned int wordNr;
                  unsigned char *s = _sampleData;
                  unsigned char *d;
                  
                  numBytesTransferred = 0;
                  dataPacketNr = 0;
                  while(numBytesTransferred < sampleByteSize)
                  {
                     // Expect SendNextPacket or wait
                     memset(&snp, 0xCD, sizeof(SMDI_SendNextPacket));
                     if(smdi_receive_message(_adapterNr, _target, (BYTE*)&snp, sizeof(SMDI_SendNextPacket)))
                     {
                        if(smdi_get_uint16(snp.id) == SMDI_MSGID_WAIT)
                        {
                           // Wait until TEST_UNIT_READY responds READY/GOOD
                           if(0==smdi_wait_unit_ready(_adapterNr, _target, SMDI_TESTUNITREADY_TIMEOUT))
                           {
                              printf("[---] tksmdi: smdi_upload_sample: Slave unit failed to become READY after SMDI_WAIT (sampleNr=%i sampleName=\"%s\")\n",
                                 _sampleNr, _sampleName);
                              return 0;
                           }
                        }
                        else if(smdi_get_uint16(snp.id) != SMDI_MSGID_SENDNEXTPACKET)
                        {
                           printf("[---] tksmdi: smdi_upload_sample: Received msgid 0x%04x instead of SENDNEXTPACKET (sampleNr=%i sampleName=\"%s\")\n",
                              smdi_get_uint16(snp.id),
                              _sampleNr, _sampleName);
                           return 0;
                        }
                        // Prepare data packet
                        memset(smdi_transfer_buffer, 0, sizeof(SMDI_DataPacket));
                        smdi_set_uint24(smdi_transfer_buffer+11, dataPacketNr);
                        d = smdi_transfer_buffer + sizeof(SMDI_DataPacket);
                        if( (numBytesTransferred+dataPacketLen) > sampleByteSize )
                        {
                           // Final packet, truncate to match remaining number of bytes
                           currentPacketLen = sampleByteSize - numBytesTransferred;
                        }
                        else
                        {
                           currentPacketLen = dataPacketLen;
                        }
                        for(wordNr=0; wordNr<(currentPacketLen/bytesPerWord); wordNr++)
                        {
                           if(_bDataIsLittleEndian)
                           {
                              switch(bytesPerWord)
                              {
                              case 1:
                                 *d++ = *s++;
                                 break;
                              case 2:
                                 *d++ = s[1];
                                 *d++ = s[0];
                                 s += 2;
                                 break;
                              case 3:
                                 *d++ = s[2];
                                 *d++ = s[1];
                                 *d++ = s[0];
                                 s += 3;
                                 break;
                              case 4:
                                 *d++ = s[3];
                                 *d++ = s[2];
                                 *d++ = s[1];
                                 *d++ = s[0];
                                 s += 4;
                                 break;
                              }
                           }
                           else
                           {
                              switch(bytesPerWord)
                              {
                              case 4:
                                 *d++ = *s++;
                              case 3:
                                 *d++ = *s++;
                              case 2:
                                 *d++ = *s++;
                              case 1:
                                 *d++ = *s++;
                                 break;
                              }
                           }
                        }

                        // Send data packet
                        if(!smdi_send_message(_adapterNr, _target, smdi_transfer_buffer, sizeof(SMDI_DataPacket)+currentPacketLen, SMDI_MSGID_DATAPACKET, 0x03+currentPacketLen))
                        {
                           printf("[---] tksmdi: smdi_upload_sample: Failed to send DataPacket (dataPacketNr=%i sampleNr=%i sampleName=\"%s\")\n",
                              dataPacketNr, _sampleNr, _sampleName);
                           return 0;
                        }

                        dataPacketNr++;
                        numBytesTransferred += currentPacketLen;
                     }
                     else
                     {
                        printf("[---] tksmdi: smdi_upload_sample: Failed to receive SendNextPacket or Wait message (sampleNr=%i sampleName=\"%s\")\n",
                           _sampleNr, _sampleName);
                        return 0;
                     }
                  } // while numTransferred<sampleByteSize

                  // Expect EndOfProcedure message
                  if(!smdi_receive_message(_adapterNr, _target, smdi_transfer_buffer, sizeof(SMDI_EndOfProcedure)))
                  {
                     printf("[---] tksmdi: smdi_upload_sample: Failed to receive EndOfProcedure message after last packet.\n");
                     return 0;
                  }
                  else
                  {
                     if(smdi_get_uint16(smdi_transfer_buffer+4) != SMDI_MSGID_ENDOFPROCEDURE)
                     {
                        printf("[---] tksmdi: smdi_upload_sample: Expected EndOfProcedure but got 0x%04x.\n",
                           smdi_get_uint16(smdi_transfer_buffer+4));
                        return 0;
                     }
                  }
                  
                  {
                     double bps;
                     tickCountEnd = GetTickCount();
                     bps = ((double)sampleByteSize) / (tickCountEnd-tickCountStart);
                     printf("[...] tksmdi: sample upload complete (%i bytes transferred; %f kb/s).\n", 
                        sampleByteSize, bps*1000.0/1024.0);
                  }
                  return 1;
               }
               else
               {
                  printf("[---] tksmdi: smdi_upload_sample: Failed to send BeginSampleTransfer message (sampleNr=%i sampleName=\"%s\")\n",
                     _sampleNr, _sampleName);
               }
            }
         }
         else
         {
            printf("[---] tksmdi: smdi_upload_sample: Receive msgid=0x%04x instead of BeginSampleTransferAcknowledge (sampleNr=%i sampleName=\"%s\")\n",
               smdi_get_uint16(bst.id),
               _sampleNr, _sampleName);
         }
      }
      else
      {
         printf("[---] tksmdi: smdi_upload_sample: Failed to receive BeginSampleTransferAcknowledge (sampleNr=%i sampleName=\"%s\")\n",
            _sampleNr, _sampleName);
      }
   }
   else
   {
      printf("[---] tksmdi: smdi_upload_sample: failed to send SampleHeader (sampleNr=%i name=\"%s\")\n",
         _sampleNr, _sampleName);
   }

   // Failed
   return 0;
}


// ---------------------------------------------------------------------------- smdi_delete_sample
int smdi_delete_sample(BYTE _adapterNr, BYTE _target, unsigned int _sampleNr) {
   SMDI_DeleteSampleFromMemory ds;

   memset(&ds,0, sizeof(SMDI_DeleteSampleFromMemory));
   smdi_set_uint24(ds.sample_nr, _sampleNr);
   if(smdi_send_message(_adapterNr, _target, (BYTE*)&ds, sizeof(SMDI_DeleteSampleFromMemory), SMDI_MSGID_DELETESAMPLEFROMMEMORY, 0x03))
   {
      union {
         SMDI_EndOfProcedure eop;
         SMDI_Reject reject;
      } msg;

      // Expect SendNextPacket or wait
      memset(&msg.eop, 0xCD, sizeof(SMDI_EndOfProcedure));
      if(smdi_receive_message(_adapterNr, _target, (BYTE*)&msg.eop, sizeof(SMDI_EndOfProcedure/*Reject*/)))
      {
         if(smdi_get_uint16(msg.eop.id) == SMDI_MSGID_WAIT)
         {
            // Wait until TEST_UNIT_READY responds READY/GOOD
            if(0==smdi_wait_unit_ready(_adapterNr, _target, SMDI_TESTUNITREADY_TIMEOUT))
            {
               printf("[---] tksmdi: smdi_delete_sample: Slave unit failed to become READY after SMDI_WAIT (adapterNr=%i target=%i sampleNr=%i)\n",
                  _adapterNr, _target, _sampleNr);
               return 0;
            }
            // Expect EndOfProcedure
            if(smdi_receive_message(_adapterNr, _target, (BYTE*)&msg.eop, sizeof(SMDI_EndOfProcedure)))
            {
               if(smdi_get_uint16(msg.eop.id) == SMDI_MSGID_ENDOFPROCEDURE)
               {
                  // OK
                  return 1;
               }
               else
               {
                  printf("[---] tksmdi: smdi_delete_sample: Received msgid 0x%04x instead of ENDOFPROCEDURE after WAIT (adapterNr=%i target=%i sampleNr=%i)\n",
                     smdi_get_uint16(msg.eop.id), 
                     _adapterNr, _target, _sampleNr);
               }
            }
         }
         else if(smdi_get_uint16(msg.eop.id) == SMDI_MSGID_REJECT)
         {
            /*printf("[---] tksmdi: smdi_delete_sample: REJECTED (sampleNr=%i code=0x%04x subcode=0x%04x)\n",
               _sampleNr, 
               smdi_get_uint16(msg.reject.rejection_code), smdi_get_uint16(msg.reject.rejection_code)
               );*/
            printf("[---] tksmdi: smdi_delete_sample: REJECTED (sampleNr=%i subid=0x%04x)\n",
               _sampleNr, 
               smdi_get_uint16(msg.eop.sub_id)
               );
         }
         else if(smdi_get_uint16(msg.eop.id) != SMDI_MSGID_ENDOFPROCEDURE)
         {
            printf("[---] tksmdi: smdi_delete_sample: Received msgid 0x%04x instead of ENDOFPROCEDURE (adapterNr=%i target=%i sampleNr=%i)\n",
               smdi_get_uint16(msg.eop.id), 
               _adapterNr, _target, _sampleNr);
         }
         else
         {
            // OK, got EndOfProcedure
            return 1;
         }
      }
      else
      {
         printf("[---] tksmdi: smdi_delete_sample: Failed to receive EndOfProcedure message (adapterNr=%i target=%i sampleNr=%i)\n",
            _adapterNr, _target, _sampleNr);
      }
   }
   else
   {
      printf("[---] tksmdi: smdi_delete_sample: failed to send DeleteSampleFromMemory message. (adapterNr=%i target=%i sampleNr=%i)\n",
         _adapterNr, _target, _sampleNr);
   }
   return 0;
}


// ---------------------------------------------------------------------------- smdi_send_midi
int smdi_send_midi(BYTE _adapterNr, BYTE _target, unsigned int _numBytes) {
   memset(smdi_transfer_buffer, 0, sizeof(SMDI_TransmitMIDIMessage));
   smdi_set_uint24(smdi_transfer_buffer + 8, _numBytes); // add_msg_len
   if(smdi_send_message(_adapterNr, _target, (BYTE*)smdi_transfer_buffer, sizeof(SMDI_TransmitMIDIMessage)+_numBytes, SMDI_MSGID_TRANSMITMIDIMESSAGE, _numBytes))
   {
      return 1;
   }
   else
   {
      printf("[---] tksmdi: smdi_send_midi: failed to send TransmitMIDIMessage (adapterNr=%i target=%i numBytes=%i).\n",
         _adapterNr, _target, _numBytes);
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_receive_midi
int smdi_receive_midi(BYTE _adapterNr, BYTE _target, unsigned int _numBytes) {
   memset(smdi_transfer_buffer, 0, sizeof(SMDI_TransmitMIDIMessage));
   if(smdi_receive_message(_adapterNr, _target, (BYTE*)smdi_transfer_buffer, sizeof(SMDI_TransmitMIDIMessage)+_numBytes))
   {
      return 1;
   }
   else
   {
      printf("[---] tksmdi: smdi_receive_midi: failed to receiveTransmitMIDIMessage (adapterNr=%i target=%i numBytes=%i).\n",
         _adapterNr, _target, _numBytes);
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_init
static int smdi_init(void) {
   FARPROC fpv;

   dll_handle = LoadLibrary("WNASPI32.DLL");
   if(NULL == dll_handle)
   {
      printf("[---] tksmdi: failed to open WNASPI32.DLL.\n");
      return 0;
   }

   // Resolve GetASPI32SupportInfo ASPI API function address
   fpv = GetProcAddress(dll_handle, "GetASPI32SupportInfo");
   if(NULL == fpv)
   {
      printf("[---] tksmdi: failed to resolve GetASPI32SupportInfo procedure.\n");
      FreeLibrary(dll_handle);
      return 0;
   }
   proc_GetASPI32SupportInfo = (proc_t_GetASPI32SupportInfo) fpv;

   // Resolve SendASPI32Command ASPI API function address
   fpv = GetProcAddress(dll_handle, "SendASPI32Command");
   if(NULL == fpv)
   {
      printf("[---] tksmdi: failed to resolve SendASPI32Command procedure.\n");
      FreeLibrary(dll_handle);
      return 0;
   }
   proc_SendASPI32Command = (proc_t_SendASPI32Command) fpv;

   // Resolve GetASPI32Buffer ASPI API function address
   fpv = GetProcAddress(dll_handle, "GetASPI32Buffer");
   if(NULL == fpv)
   {
      printf("[---] tksmdi: failed to resolve GetASPI32Buffer procedure.\n");
      FreeLibrary(dll_handle);
      return 0;
   }
   proc_GetASPI32Buffer = (proc_t_GetASPI32Buffer) fpv;

   // Resolve FreeASPI32Buffer ASPI API function address
   fpv = GetProcAddress(dll_handle, "FreeASPI32Buffer");
   if(NULL == fpv)
   {
      printf("[---] tksmdi: failed to resolve FreeASPI32Buffer procedure.\n");
      FreeLibrary(dll_handle);
      return 0;
   }
   proc_FreeASPI32Buffer = (proc_t_FreeASPI32Buffer) fpv;

   // Resolve TranslateASPI32Address ASPI API function address
   fpv = GetProcAddress(dll_handle, "TranslateASPI32Address");
   if(NULL == fpv)
   {
      printf("[---] tksmdi: failed to resolve TranslateASPI32Address procedure.\n");
      FreeLibrary(dll_handle);
      return 0;
   }
   proc_TranslateASPI32Address = (proc_t_TranslateASPI32Address) fpv;

   // Create event handle
   event_handle = CreateEvent ( NULL, 1, 0, NULL );
   if(NULL == event_handle)
   {
      printf("[---] tksmdi: unable to create event object.\n");
      FreeLibrary(dll_handle);
      return 0;
   }

   // Ok
   if(proc_GetASPI32SupportInfo())
   {

      // Query host adapters
      get_hostadapter_info();

      return 1;
   }
   else
   {
      tkscript->printf("[---] tksmdi: GetASPI32SupportInfo() failed.\n");
      return 0;
   }
}


// ---------------------------------------------------------------------------- smdi_exit
static void smdi_exit(void) {
   CloseHandle(event_handle);
   FreeLibrary(dll_handle);
}


// ---------------------------------------------------------------------------- SMDI
YC class SMDI : public YAC_Object {
};


// ---------------------------------------------------------------------------- YAC_Init
void YAC_CALL YAC_Init(YAC_Host *_host) {
	yac_host = _host;

	if(yac_host->yacGetDebugLevel()) 
	  yac_host->printf("tksmdi::YAC_Init called.\n");

   YAC_Init_smdi(_host);

}


// ---------------------------------------------------------------------------- YAC_Exit
void YAC_CALL YAC_Exit(YAC_Host *_host) {
   YAC_Exit_smdi(_host);
}


// ---------------------------------------------------------------------------- YAC_Version
sUI YAC_CALL YAC_Version(void) {
	return 0x00010001;
}


// ---------------------------------------------------------------------------- main
#ifdef HAVE_MAIN
int main(int _argc, char **_argv) {

   if(smdi_init())
   {

#ifdef DO_RESCAN_SCSI_BUS 
      rescan_scsi_bus(adapter_nr);
#endif // DO_RESCAN_SCSI_BUS

#ifdef DO_SCSI_INQUIRY
      {
         BYTE target;
         for(target=0; target<8; target++)
         {
            scsi2_inquiry(adapter_nr, target);
         }
      }
#endif // DO_SCSI_INQUIRY

#ifdef DO_PRINT_SAMPLEHEADERS
      {
         unsigned int sampleNr;
         for(sampleNr=0; sampleNr<10/*127*/; sampleNr++)
         {
            smdi_print_sampleheader(adapter_nr, target_nr, sampleNr);
         }
      }
#endif // DO_PRINT_SAMPLEHEADERS


#ifdef DO_SAMPLE_DOWNLOAD
      if(smdi_download_sample(adapter_nr, target_nr, 7))
      {
         printf("yippie\n");
      }
#endif // DO_SAMPLE_DOWNLOAD


#ifdef DO_SAMPLE_UPLOAD
      {
         // Calculate test sample and upload it
         short *test_sample;
         unsigned int i;
         double ang;
         double t;
#define TEST_SAMPLE_LEN 128
         
         test_sample = (short*) malloc(2*TEST_SAMPLE_LEN);
         ang = 0.0;
         for(i=0; i<TEST_SAMPLE_LEN; i++)
         {
            t = sin(ang);
            test_sample[i] = (short) (t*32767.0);
            //test_sample[i] = (short) (-32767 + (i*(65535/TEST_SAMPLE_LEN)));
            ang += (2*3.14159265358979323)/TEST_SAMPLE_LEN;
         }
         if(smdi_upload_sample(adapter_nr, target_nr, 8, 16, 1, (unsigned int)(1000000000.0/44100.0),
            TEST_SAMPLE_LEN, 0, TEST_SAMPLE_LEN-1, 0x00, 60, 0, "test_sinewave", (unsigned char*)test_sample, 1))
         {
            printf("yippie2\n");
            smdi_print_sampleheader(adapter_nr, target_nr, 8);

#ifdef DO_DELETE_SAMPLE
            // Not supported on Yamaha A5000 :(
            Sleep(4000);
            smdi_delete_sample(adapter_nr, target_nr, 8);
#endif

            smdi_print_sampleheader(adapter_nr, target_nr, 8);
         }
         free(test_sample);
      }
#endif // DO_SAMPLE_UPLOAD

#ifdef DO_MIDI_SYSEX
      {
         // Send Yamaha A5000 SystemParameters bulk dump request
         unsigned char *d = smdi_transfer_buffer + sizeof(SMDI_TransmitMIDIMessage);
         *d++ = 0xF0;
         *d++ = 0x43;
         *d++ = 0x20 | 0; // DeviceNr
         *d++ = 0x7A;
         *d++ = 'L';
         *d++ = 'M';
         *d++ = ' ';
         *d++ = ' ';
         *d++ = '0';
         *d++ = '4';
         *d++ = '7';
         *d++ = '5'; // '4' for A4000, '5' for A5000
         *d++ = 'S';
         *d++ = 'Y';
         *d++ = 'S';
         *d++ = 'y';
         *d++ = 's';
         *d++ = 't';
         *d++ = 'e';
         *d++ = 'm';
         *d++ = 'P';
         *d++ = 'a';
         *d++ = 'r';
         *d++ = 'a';
         *d++ = 'm';
         *d++ = 'e';
         *d++ = 't';
         *d++ = 'e';
         *d++ = 'r';
         *d++ = 's';
         *d++ = 0xF7;
         //smdi_send_midi(adapter_nr, target_nr, 31);

         // Select Yamaha A5000 F-Key
         d = smdi_transfer_buffer + sizeof(SMDI_TransmitMIDIMessage);
         *d++ = 0xF0; *d++ = 0x43; *d++ = 0x10; *d++ = 0x58; *d++ = 0x03; *d++ = 0x04;
         *d++ = 0x00; *d++ = 0x00; *d++ = 0x00; *d++ = 0x00; *d++ = 0x00; *d++ = 0x7F; *d++ = 0xF7;
         smdi_send_midi(adapter_nr, target_nr, 13);
      }

#endif // DO_MIDI_SYSEX


      smdi_exit();
   }
   else
   {
      printf("[...] tksmdi: init failed.\n");
   }

   return 0;
}

#endif // HAVE_MAIN

